SystemOrganization addCategory: 'Metacello-Platform'!

!ExecutableBlock methodsFor: '*metacello-platform' stamp: 'DaleHenrichs 01/20/2010 11:16'!
valueSupplyingMetacelloAnswers: aListOfPairs
	"evaluate the block using a list of questions / answers that might be called upon to
	automatically respond to Object>>confirm: or FillInTheBlank requests"

	^ [self value] 
		on: ProvideAnswerNotification
		do: 
			[:notify | | answer caption |
			
			caption := notify messageText withSeparatorsCompacted. "to remove new lines"
			answer := aListOfPairs
				detect: 
					[:each | caption = each first or:
						[(caption includesSubstring: each first caseSensitive: false) or:
						[(each first match: caption) or:
						[(String includesSelector: #matchesRegex:) and: 
						[ [ caption matchesRegex: each first ] on: Error do: [:ignored | false ]]]]]]
					ifNone: [nil].
			answer
				ifNotNil: [notify resume: answer second]
				ifNil: 
					[ | outerAnswer |
					outerAnswer := ProvideAnswerNotification signal: notify messageText.
					outerAnswer 
						ifNil: [notify resume] 
						ifNotNil: [notify resume: outerAnswer]]]! !

!ClientFileDirectory methodsFor: '*metacello-platform' stamp: 'dkh 01/19/2012 12:10'!
asRepositorySpecFor: aMetacelloMCProject

	^(aMetacelloMCProject repositorySpec)
		description:  'client://', self fullName;
	 	type: 'clientDirectory';
		yourself! !

!FileDirectory methodsFor: '*metacello-platform' stamp: 'dkh 02/25/2013 16:14'!
asRepositorySpecFor: aMetacelloMCProject
  ""

  ^ aMetacelloMCProject repositorySpec
    description: self fullName;
    type: 'directory';
    yourself! !

MetacelloPlatform subclass: #MetacelloGemStonePlatform
	instanceVariableNames: 'stoneVersionAttribute'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Metacello-Platform'!

!MetacelloGemStonePlatform class methodsFor: 'initialize-release' stamp: 'dkh 05/05/2013 13:40'!
initialize
  self select! !

!MetacelloGemStonePlatform methodsFor: 'utilities' stamp: 'dkh 11/10/2011 09:52'!
authorName

	^MCPlatformSupport authorInitials! !

!MetacelloGemStonePlatform methodsFor: 'utilities' stamp: 'dkh 11/10/2011 09:52'!
authorName: aString

	^GsPackagePolicy current authorInitials: aString! !

!MetacelloGemStonePlatform methodsFor: 'user interaction' stamp: 'dkh 01/06/2010 10:18'!
confirm: aString
	
	Smalltalk 
		at: #OBGemStonePlatform 
		ifPresent: [:cl | 
			(cl perform: #canConfirm)
				ifTrue: [
					Smalltalk 
						at: #OBConfirmationRequest 
						ifPresent: [:obClass |
							^obClass perform: #prompt:  with: aString ]]].
	Warning signal: aString.
	^true! !

!MetacelloGemStonePlatform methodsFor: 'reflection' stamp: 'dkh 11/10/2011 09:52'!
copyClass: oldClass as: newName inCategory: newCategoryName

	| definition newClass |
	Smalltalk at: newName asSymbol ifPresent: [^ self error: newName , ' already exists'].
	definition := oldClass 
						definitionWithClassName: newName asSymbol
						superClassName: (oldClass superClass == nil 
							ifTrue: ['nil'] ifFalse: [oldClass superClass name]).
	definition := definition copyReplaceAll: 'category: ', oldClass category asString printString with: 'category: ', newCategoryName printString.
	newClass := definition evaluate.
	newClass 
		copyMethodsFrom: oldClass 
		dictionaries: GsSession currentSession symbolList.
	^ newClass	! !

!MetacelloGemStonePlatform methodsFor: 'repository creation' stamp: 'dkh 03/23/2013 08:50'!
createRepository: aRepositorySpec
  | type headerSize description |
  type := aRepositorySpec type.
  type = 'directory'
    ifTrue: [ 
      ^ MCServerDirectoryRepository new
        directory: (ServerFileDirectory on: aRepositorySpec description) ].
  type = 'clientDirectory'
    ifTrue: [ 
      headerSize := 'client://' size.
      description := aRepositorySpec description.
      ^ MCDirectoryRepository new
        directory:
          (ClientFileDirectory
            on: (description copyFrom: headerSize + 1 to: description size)) ].
  type = 'serverDirectory'
    ifTrue: [ 
      headerSize := 'server://' size.
      description := aRepositorySpec description.
      ^ MCServerDirectoryRepository new
        directory:
          (ServerFileDirectory
            on: (description copyFrom: headerSize + 1 to: description size)) ].
  type = 'cache'
    ifTrue: [ 
      | description headerSize atLocation oop |
      description := aRepositorySpec description.
      headerSize := 'cache://' size.
      atLocation := description indexOf: $@.
      oop := (description copyFrom: headerSize + 1 to: atLocation - 1) asNumber.
      ^ Object _objectForOop: oop ].
  Smalltalk
    at: #'MCFileTreeRepository'
    ifPresent: [ :cl | 
      type = 'filetree'
        ifTrue: [ 
          | description headerSize |
          description := aRepositorySpec description.
          headerSize := 'filetree://' size.
          ^ cl new
            directory:
              (ServerFileDirectory
                on:
                  (aRepositorySpec description copyFrom: headerSize + 1 to: description size)) ] ].
  ^ super createRepository: aRepositorySpec! !

!MetacelloGemStonePlatform methodsFor: 'attributes' stamp: 'dkh 11/13/2009 16:37'!
defaultPlatformAttributes

	| stoneVersionAtts |
	(stoneVersionAtts := self stoneVersionAttributes) ifNil: [^ #( #gemstone )].
	^ stoneVersionAtts! !

!MetacelloGemStonePlatform methodsFor: 'github support' stamp: 'dkh 09/26/2014 14:23'!
downloadFile: url to: outputFileName
  "download from <url> into <outputFileName>"

  | archive zipfile errorFileName stream |
  "Make a unique name for the error log and ensure that we can write the files"
  errorFileName := self downloadErrorFileNameFor: outputFileName.
  (ServerFileDirectory on: outputFileName) forceNewFileNamed: outputFileName.
  (ServerFileDirectory on: errorFileName) forceNewFileNamed: errorFileName.
  System
    performOnServer:
      '/usr/bin/curl -L ' , url , ' > ' , outputFileName , ' 2> ' , errorFileName.
  archive := ZipArchive new.
  zipfile := ServerFileDirectory on: outputFileName.
  zipfile containingDirectory
    readOnlyFileNamed: zipfile localName
    do: [ :fileStream | 
      stream := RWBinaryOrTextStream on: fileStream contents ].
  [ archive readFrom: stream ]
    on: Error
    do: [ :ex | 
      | dir curlErrorString |
      dir := ServerFileDirectory on: errorFileName.
      (dir fileExists: errorFileName)
        ifTrue: [ 
          dir
            readOnlyFileNamed: errorFileName
            do: [ :fileStream | 
              curlErrorString := '. Error could have been caused by: '
                , fileStream contents printString ] ]
        ifFalse: [ curlErrorString := '.' ].
      self
        error:
          'Error during download: ' , ex description printString , curlErrorString ].
  (ServerFileDirectory on: '/tmp') deleteFileNamed: errorFileName.
  ^ archive! !

!MetacelloGemStonePlatform methodsFor: 'attributes' stamp: 'dkh 11/10/2011 09:52'!
extractAtrributesFrom: gsVersion

	| tokens attributes attribute |
	attributes := { #gemstone }.
	tokens := gsVersion subStrings: $..
	attribute := ''.
	1 to: tokens size do: [:index | | token |
		(token := tokens at: index) isEmpty ifTrue: [ ^attributes ].
		attribute := attribute, token.
		attributes add: ('gs', attribute, '.x') asSymbol.
		attribute := attribute, '.'].
	^attributes! !

!MetacelloGemStonePlatform methodsFor: 'github support' stamp: 'dkh 07/01/2013 08:33'!
extractRepositoryFrom: zipFile to: directory
  "unzip <zipFile> into <directory>"

  | errorMessage |
  (ServerFileDirectory on: directory) assureExistence.
  System
    performOnServer:
      '/usr/bin/unzip -u ' , zipFile , ' -d ' , directory , ' 2> /tmp/zip.err'.
  (ServerFileDirectory on: '/tmp')
    fileNamed: 'zip.err'
    do: [ :fileStream | 
      (errorMessage := fileStream contentsOfEntireFile) notEmpty
        ifTrue: [ self error: 'unzip failure: ' , errorMessage printString ]
        ifFalse: [ 
          (ServerFileDirectory on: '/tmp')
            deleteFileNamed: zipFile;
            deleteFileNamed: (self downloadErrorFileNameFor: zipFile) ] ]! !

!MetacelloGemStonePlatform methodsFor: 'repository creation' stamp: 'dkh 03/23/2013 08:54'!
extractTypeFromDescription: description
  description == nil
    ifTrue: [ ^ nil ].
  (description beginsWith: 'client://')
    ifTrue: [ ^ 'clientDirectory' ].
  (description beginsWith: 'server://')
    ifTrue: [ ^ 'serverDirectory' ].
  (description beginsWith: 'cache://')
    ifTrue: [ ^ 'cache' ].
  ^ super extractTypeFromDescription: description! !

!MetacelloGemStonePlatform methodsFor: 'github support' stamp: 'dkh 07/09/2012 04:46'!
fileDirectoryClass

	^ServerFileDirectory! !

!MetacelloGemStonePlatform methodsFor: 'attributes' stamp: 'dkh 11/10/2011 09:52'!
stoneVersionAttributes

	stoneVersionAttribute isSymbol ifTrue: [ stoneVersionAttribute := nil ].
	stoneVersionAttribute ifNil: [ | atts |
				stoneVersionAttribute := self extractAtrributesFrom: (System stoneVersionAt: 'gsVersion').
				(Globals at: #RubySystemExit otherwise:nil) ~~ nil
					ifTrue: [ stoneVersionAttribute add: #'maglev' ]].
	^ stoneVersionAttribute! !

!MetacelloGemStonePlatform methodsFor: 'utilities' stamp: 'dkh 10/16/2009 11:53'!
timestamp

	^TimeStamp now printString! !

!MetacelloGemStonePlatform methodsFor: 'transactions' stamp: 'dkh 01/06/2010 10:11'!
transact: aBlock
	"On GemStone, we want to optionally abort before command execution and commit after 
	 common execution. Other plaforms don't need to do anything special.
	 Returning out of block, skips commit."
	
	MCPlatformSupport autoAbort ifTrue: [ System abortTransaction ].
	aBlock value.
	MCPlatformSupport autoCommit
		ifTrue: [
			System commitTransaction
				ifFalse: [
					Transcript cr; show: 'Auto commit failed.'.
					Smalltalk at: #OBInformRequest ifPresent: [:cls | 
						cls message: 'Auto commit failed. You will need to abort and try again.' ]]].! !
MetacelloGemStonePlatform initialize!
